@@ -7,7 +7,7 @@ options(prompt="R> ", width=77, digits=4, useFancyQuotes=FALSE)
 
 
 ###################################################
-### code chunk number 2: synthpop.Rnw:154-155
+### code chunk number 2: synthpop.Rnw:160-161
 ###################################################
 library("synthpop")
 
@@ -21,50 +21,50 @@ head(ods)
 
 
 ###################################################
-### code chunk number 4: synthpop.Rnw:186-188
+### code chunk number 4: synthpop.Rnw:192-194
 ###################################################
 my.seed <- 17914709
 sds.default <- syn(ods, seed = my.seed)
 
 
 ###################################################
-### code chunk number 5: synthpop.Rnw:193-194
+### code chunk number 5: synthpop.Rnw:199-200
 ###################################################
 sds.default
 
 
 ###################################################
-### code chunk number 6: synthpop.Rnw:199-200
+### code chunk number 6: synthpop.Rnw:205-206
 ###################################################
 names(sds.default)
 
 
 ###################################################
-### code chunk number 7: synthpop.Rnw:209-210
+### code chunk number 7: synthpop.Rnw:215-216
 ###################################################
 sds.parametric <- syn(ods, method = "parametric", seed = my.seed)
 
 
 ###################################################
-### code chunk number 8: synthpop.Rnw:212-213
+### code chunk number 8: synthpop.Rnw:218-219
 ###################################################
 sds.parametric$method
 
 
 ###################################################
-### code chunk number 9: synthpop.Rnw:223-224
+### code chunk number 9: synthpop.Rnw:229-230
 ###################################################
 sds.selection <- syn(ods, visit.sequence = c(1, 2, 6, 4, 3), seed = my.seed)
 
 
 ###################################################
-### code chunk number 10: synthpop.Rnw:229-230
+### code chunk number 10: synthpop.Rnw:235-236
 ###################################################
 sds.selection
 
 
 ###################################################
-### code chunk number 11: synthpop.Rnw:245-249
+### code chunk number 11: synthpop.Rnw:251-255
 ###################################################
 visit.sequence.ini <- c(1, 2, 5, 6, 4, 3)
 method.ini <- c("sample", "ctree", "ctree", "polyreg", "", "ctree", "")
@@ -73,7 +73,7 @@ sds.ini <- syn(data = ods, visit.sequence = visit.sequence.ini,
 
 
 ###################################################
-### code chunk number 12: synthpop.Rnw:251-255
+### code chunk number 12: synthpop.Rnw:257-261
 ###################################################
 sds.ini$predictor.matrix
 predictor.matrix.corrected <- sds.ini$predictor.matrix
@@ -82,7 +82,7 @@ predictor.matrix.corrected
 
 
 ###################################################
-### code chunk number 13: synthpop.Rnw:257-260
+### code chunk number 13: synthpop.Rnw:263-266
 ###################################################
 sds.corrected <- syn(data = ods, visit.sequence = visit.sequence.ini,
   method = method.ini, predictor.matrix = predictor.matrix.corrected,
@@ -90,50 +90,49 @@ sds.corrected <- syn(data = ods, visit.sequence = visit.sequence.ini,
 
 
 ###################################################
-### code chunk number 14: synthpop.Rnw:267-269
+### code chunk number 14: synthpop.Rnw:272-274
 ###################################################
-cont.na.income <- as.list(rep(NA, ncol(ods)))
-cont.na.income[[5]] <- c(NA, -8)
+sds.income <- syn(ods, cont.na = list(income = c(NA, -8)), 
+  smoothing = list(income = "density"), seed = NA)
 
 
 ###################################################
-### code chunk number 15: synthpop.Rnw:275-282
+### code chunk number 15: synthpop.Rnw:280-288
 ###################################################
-maritalM18.ods <- table(ods[ods$age < 18 & ods$sex == 'MALE', "marital"])
-maritalM18.default <- table(sds.default$syn[sds.default$syn$age < 18 &
-  sds.default$syn$sex == 'MALE', "marital"])
-maritalM18.parametric <- table(sds.parametric$syn[sds.default$syn$age < 18 &
-  sds.parametric$syn$sex == 'MALE', "marital"])
-cbind("Observed data" = maritalM18.ods, CART = maritalM18.default,
-  Parametric = maritalM18.parametric)
+M18.ods <- table(subset(ods,
+  age < 18 & sex == 'MALE', marital))
+M18.default <- table(subset(sds.default$syn,
+  age < 18 & sex == 'MALE', marital))
+M18.parametric <- table(subset(sds.parametric$syn,
+  age < 18 & sex == 'MALE', marital))
+cbind("Observed data" = M18.ods, CART = M18.default,
+  Parametric = M18.parametric)
 
 
 ###################################################
-### code chunk number 16: synthpop.Rnw:286-298
+### code chunk number 16: synthpop.Rnw:292-298
 ###################################################
-rules.marital <- list("", "", "", "age < 18 & sex == 'MALE'", "", "", "")
-rvalues.marital <- list(NA, NA, NA, 'SINGLE', NA, NA, NA)
-sds.rmarital <- syn(ods, rules = rules.marital, 
+rules.marital <- list(marital = "age < 18 & sex == 'MALE'")
+rvalues.marital <- list(marital = 'SINGLE')
+sds.rmarital <- syn(ods, rules = rules.marital,
   rvalues = rvalues.marital, seed = my.seed)
-sds.rmarital.param <- syn(ods, rules = rules.marital, 
+sds.rmarital.param <- syn(ods, rules = rules.marital,
   rvalues = rvalues.marital, method = "parametric", seed = my.seed)
 
-rmaritalM18.default <- table(sds.rmarital$syn[sds.rmarital$syn$age < 18
-  & sds.rmarital$syn$sex == 'MALE', "marital"])
-rmaritalM18.parametric <- table(sds.rmarital.param$syn[
-  sds.rmarital.param$syn$age < 18
-  & sds.rmarital.param$syn$sex == 'MALE', "marital"])
+
+###################################################
+### code chunk number 17: synthpop.Rnw:301-307
+###################################################
+rM18.default <- table(subset(sds.rmarital$syn,
+  age < 18 & sex == 'MALE', marital))
+rM18.parametric <- table(subset(sds.rmarital.param$syn,
+  age < 18 & sex == 'MALE', marital))
+cbind("Observed data" = M18.ods, CART = rM18.default,
+  Parametric = rM18.parametric)
 
 
 ###################################################
-### code chunk number 17: synthpop.Rnw:300-302
-###################################################
-cbind("Observed data" = maritalM18.ods, CART = rmaritalM18.default,
-  Parametric = rmaritalM18.parametric) 
-
-
-###################################################
-### code chunk number 18: synthpop.Rnw:310-315
+### code chunk number 18: synthpop.Rnw:314-319
 ###################################################
 ods$wkabint <- as.character(ods$wkabint)
 ods$wkabint[ods$wkabint == 'YES, TO EU COUNTRY' |
@@ -143,75 +142,63 @@ ods$income[ods$income == -8] <- NA
 
 
 ###################################################
-### code chunk number 19: synthpop.Rnw:320-321
+### code chunk number 19: synthpop.Rnw:324-325
 ###################################################
 sds <- syn(ods, m = 5, seed = my.seed)
 
 
 ###################################################
-### code chunk number 20: synthpop.Rnw:326-327
+### code chunk number 20: synthpop.Rnw:330-331
 ###################################################
 summary(ods)
 
 
 ###################################################
-### code chunk number 21: synthpop.Rnw:332-333
+### code chunk number 21: synthpop.Rnw:336-337
 ###################################################
 summary(sds)
 
 
 ###################################################
-### code chunk number 22: synthpop.Rnw:335-337
+### code chunk number 22: synthpop.Rnw:342-344
 ###################################################
 summary(sds, msel = 2)
 summary(sds, msel = 1:5)
 
 
 ###################################################
-### code chunk number 23: synthpop.Rnw:342-343 (eval = FALSE)
+### code chunk number 23: synthpop.Rnw:349-350
 ###################################################
-## compare.synds(sds, ods)
+compare(sds, ods, vars = "income")  
 
 
 ###################################################
-### code chunk number 24: synthpop.Rnw:345-346
+### code chunk number 24: synthpop.Rnw:361-362
 ###################################################
-compare.synds(sds, ods, vars = "ls")
+compare(sds, ods, vars = "ls", msel = 1:3)
 
 
 ###################################################
-### code chunk number 25: synthpop.Rnw:355-356
-###################################################
-compare.synds(sds, ods, vars = "income")  
-
-
-###################################################
-### code chunk number 26: synthpop.Rnw:366-373
+### code chunk number 25: synthpop.Rnw:373-380
 ###################################################
 model.ods <- glm(wkabint ~ sex + age + edu + log(income), 
   family = "binomial", data = ods) 
-summary(model.ods)
+model.ods
                              
 model.sds <- glm.synds(wkabint ~ sex + age + edu + log(income), 
-  family = "binomial", object = sds) 
+  family = "binomial", data = sds) 
 model.sds
 
 
 ###################################################
-### code chunk number 27: synthpop.Rnw:376-377
-###################################################
-print(model.sds, msel = 3)
-
-
-###################################################
-### code chunk number 28: synthpop.Rnw:384-385
+### code chunk number 26: synthpop.Rnw:387-388
 ###################################################
 summary(model.sds)
 
 
 ###################################################
-### code chunk number 29: synthpop.Rnw:390-391
+### code chunk number 27: synthpop.Rnw:393-394
 ###################################################
-compare.fit.synds(model.sds, ods)
+compare(model.sds, ods)
 
 
